package xyz.lychee.lagfixer.libs.adventure.text.serializer.legacy;

import java.util.Objects;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.text.format.NamedTextColor;
import xyz.lychee.lagfixer.libs.adventure.text.format.TextColor;
import xyz.lychee.lagfixer.libs.adventure.text.format.TextDecoration;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

public final class LegacyFormat implements Examinable {
   static final LegacyFormat RESET = new LegacyFormat(true);
   private final @Nullable NamedTextColor color;
   private final @Nullable TextDecoration decoration;
   private final boolean reset;

   LegacyFormat(final @Nullable NamedTextColor color) {
      this.color = color;
      this.decoration = null;
      this.reset = false;
   }

   LegacyFormat(final @Nullable TextDecoration decoration) {
      this.color = null;
      this.decoration = decoration;
      this.reset = false;
   }

   private LegacyFormat(final boolean reset) {
      this.color = null;
      this.decoration = null;
      this.reset = reset;
   }

   public @Nullable TextColor color() {
      return this.color;
   }

   public @Nullable TextDecoration decoration() {
      return this.decoration;
   }

   public boolean reset() {
      return this.reset;
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (other != null && this.getClass() == other.getClass()) {
         LegacyFormat that = (LegacyFormat)other;
         return this.color == that.color && this.decoration == that.decoration && this.reset == that.reset;
      } else {
         return false;
      }
   }

   public int hashCode() {
      int result = Objects.hashCode(this.color);
      result = 31 * result + Objects.hashCode(this.decoration);
      result = 31 * result + Boolean.hashCode(this.reset);
      return result;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("color", this.color), ExaminableProperty.of("decoration", this.decoration), ExaminableProperty.of("reset", this.reset));
   }
}
